/**
 * @file
 * A Backbone Model for the edit_ui block.
 */

(function (Drupal, Backbone) {

  "use strict";

  /**
   * Backbone model for the edit_ui block.
   */
  Drupal.editUiBlock.BlockModel = Backbone.Model.extend({
    defaults: {
      content: null,
      plugin_id: false,
      region: false,
      weight: 0,
      label: '',
      html_id: 0,
      block: null,
      ghost: null,
      isVisible: false,
      isDragging: false,
      offset: {
        top: 0,
        left: 0
      },
      dimensions: {
        width: 0,
        height: 0
      },
      margins: {
        top: 0,
        bottom: 0
      }
    },

    /**
     * {@inheritdoc}
     */
    urlRoot: Drupal.url('edit-ui/block'),

    /**
     * {@inheritdoc}
     */
    toJSON: function () {
      return {
        region: this.get('region'),
        weight: this.get('weight')
      };
    },

    /**
     * Set block visible state.
     */
    show: function () {
      this.set({isVisible: true});
    },

    /**
     * Set block visible state.
     */
    hide: function () {
      this.set({isVisible: false});
    },

    /**
     * Set block element.
     *
     * @param jQuery $block
     *   Block element.
     */
    setBlock: function ($block) {
      this.set({block: $block});
    },

    /**
     * Set ghost element.
     *
     * @param jQuery $ghost
     *   Ghost block element.
     */
    setGhost: function ($ghost) {
      this.set({ghost: $ghost});
    },

    /**
     * Set block dragging state.
     */
    startDrag: function () {
      this.set({isDragging: true});
    },

    /**
     * Set block not dragging state.
     */
    stopDrag: function () {
      this.set({isDragging: false});
    },

    /**
     * Drop block in a new region.
     *
     * @param String region
     *   The region identifier.
     * @param Number weight
     *   The weight of the block within the region.
     */
    drop: function (region, weight) {
      if (!this.isNew()) {
        this.save(
          {
            region: region,
            weight: weight
          },
          {
            wait: true
          }
        );
      }
      else {
        this.set({
          region: region,
          weight: weight
        });
      }
    },

    /**
     * Update weight of a block and save it.
     *
     * @param Number weight
     *   The weight of the block within the region.
     */
    setWeight: function (weight) {
      this.set({weight: weight});
      if (!this.isNew()) {
        this.save();
      }
    },

    /**
     * Set block offset.
     *
     * @param Object offset
     *   Block offset.
     */
    setOffset: function (offset) {
      this.set({offset: offset});
    },

    /**
     * Set block dimensions.
     *
     * @param Object dimensions
     *   Block dimensions.
     */
    setDimensions: function (dimensions) {
      this.set({dimensions: dimensions});
    },

    /**
     * Set block margins.
     *
     * @param Object margins
     *   Block margins.
     */
    setMargins: function (margins) {
      this.set({margins: margins});
    }
  });

}(Drupal, Backbone));
