/**
 * @file
 * A Backbone view for the edit_ui body element.
 */

(function (Drupal, Backbone, $, Modernizr) {
  "use strict";

  /**
   * Backbone view for the edit_ui body.
   */
  Drupal.editUiBlock.BodyVisualView = Backbone.View.extend({
    /**
     * Custom data.
     */
    draggingClass: 'is-edit-ui-body-dragging',

    /**
     * Main element.
     */
    el: 'body',

    /**
     * Dom elements events.
     */
    events: {
      "startDrag": "startDrag"
    },

    /**
     * {@inheritdoc}
     */
    initialize: function (options) {
      // Add listeners.
      this.listenTo(this.collection, 'add', this.blockAdded);
      this.listenTo(this.collection, 'change:isDragging', this.render);

      // Initialize collection.
      this.collection.fetch();
    },

    /**
     * {@inheritdoc}
     */
    remove: function () {
      this.$el.off('.editUiBlockBodyVisualView');
      $(document).off('.editUiBlockBodyVisualView');
      Backbone.View.prototype.remove.apply(this, arguments);
    },

    /**
     * {@inheritdoc}
     */
    render: function (block) {
      this.$el.toggleClass(this.draggingClass, block.get('isDragging'));
    },

    /**
     * Block added callback (called when block has been added to collection).
     *
     * @param Drupal.editUiBlock.BlockModel block
     *   Block model instance.
     */
    blockAdded: function (block) {
      var region = Drupal.editUiRegion.collections.regionCollection.getRegion(block.get('region'));

      // Region is empty when a block is created from the toolbar.
      if (!region || block.get('plugin_id') === 'system_main_block') {
        return;
      }

      if ($('#' + block.get('html_id')).length === 0) {
        // Wait other blocks initializes and create a placeholder block.
        window.setTimeout(
          $.proxy(this.initBlock, this, block),
          0
        );
      }
      else {
        this.initViews(block, $('#' + block.get('html_id')));
      }
    },

    /**
     * Initializes a block and add it to the region.
     *
     * @param Drupal.editUiBlock.BlockModel block
     *   Block model instance.
     * @param jQuery $el
     *   Block DOM jQuery object.
     */
    initBlock: function (block, $el) {
      if (!$el || $el.length === 0) {
        // Create a placeholder block.
        $el = Drupal.theme('editUiPlaceholderBlock', block.attributes);
        $el = $(Drupal.theme('editUiWrapperBlock', block.attributes, $el));
      }

      // Add block into region.
      Drupal.editUiRegion.collections.regionCollection
        .getRegion(block.get('region'))
        .trigger('addBlock', block, $el);

      // Other initilalizations.
      Drupal.attachBehaviors($el.get(0));
      this.initViews(block, $el);
    },

    /**
     * Init views for block.
     *
     * @param Drupal.editUiBlock.BlockModel block
     *   Block model instance.
     * @param jQuery $el
     *   Block DOM jQuery object.
     */
    initViews: function (block, $el) {
      var view;

      // Init the block view.
      view = new Drupal.editUiBlock.BlockVisualView({
        el: $el,
        model: block
      });
      Drupal.editUiBlock.views.blockVisualViews.push(view);

      // Init the ghost block view.
      view = new Drupal.editUiBlock.GhostBlockVisualView({
        el: $(Drupal.theme('editUiGhostBlock')),
        model: block
      });
      Drupal.editUiBlock.views.ghostBlockVisualViews.push(view);
    },

    /**
     * Start drag.
     *
     * @param Event event
     *   The event object.
     * @param Drupal.editUiBlock.BlockModel block
     *   The block model of the dragged element.
     * @param Object position
     *   The mouse or touch position.
     * @param Object dimensions
     *   Original element dimensions.
     * @param Object offset
     *   Original element offset.
     */
    startDrag: function (event, block, position, dimensions, offset) {
      // Select start region.
      var region = Drupal.editUiRegion.collections.regionCollection.getRegion(block.get('region'));
      if (region) {
        region.select();
      }

      // Add listeners.
      if (Modernizr.touch) {
        this.$el.on('touchmove.editUiBlockBodyVisualView', $.proxy(this.drag, this, block));
        this.$el.on('touchend.editUiBlockBodyVisualView', $.proxy(this.drop, this, block));
      }
      else {
        this.$el.on('mousemove.editUiBlockBodyVisualView', $.proxy(this.drag, this, block));
        this.$el.on('mouseup.editUiBlockBodyVisualView', $.proxy(this.drop, this, block));
      }
    },

    /**
     * Drag.
     *
     * @param Drupal.editUiBlock.BlockModel block
     *   The dropped block.
     * @param Event event
     *   The event object.
     */
    drag: function (block, event) {
      var position;
      event.preventDefault();

      // Dragging block styles.
      position = Drupal.editUiUtils.getPosition(event);
      Drupal.editUiBlock.models.draggingBlockModel.setOffset({
        top: position.y,
        left: position.x
      });

      // Propagates drag events to region collection.
      Drupal.editUiRegion.collections.regionCollection.drag(block, position.x, position.y);
    },

    /**
     * Drop.
     *
     * @param Drupal.editUiBlock.BlockModel block
     *   The dropped block.
     * @param Event event
     *   The event object.
     */
    drop: function (block, event) {
      // Propagates drop events to region collection.
      var position = Drupal.editUiUtils.getPosition(event);
      Drupal.editUiRegion.collections.regionCollection.drop(block, position.x, position.y);

      // Remove listeners.
      this.$el.off('.editUiBlockBodyVisualView');

      // Update model state.
      block.stopDrag();

      // Trigger stopDrag event.
      Drupal.editUiBlock.models.draggingBlockModel.trigger('stopDrag', block, position);

      // Reset default states.
      Drupal.editUiUtils.reset();
    }
  });

}(Drupal, Backbone, jQuery, Modernizr));
