/**
 * @file
 * Drupal behavior for the edit_ui blocks.
 */

(function (Drupal, $) {
  "use strict";

  /**
   * edit_ui block Backbone objects.
   */
  Drupal.editUiUtils = {

    /**
     * Get mouse or touch position.
     *
     * @param Event event
     *   The event object.
     */
    getPosition: function (event) {
      var position = {x: 0, y: 0};

      if (event.type.indexOf('touch') === 0) {
        position = {
          x: event.originalEvent.changedTouches[0].pageX,
          y: event.originalEvent.changedTouches[0].pageY
        };
      }
      else {
        position = {
          x: event.pageX,
          y: event.pageY
        };
      }

      return position;
    },

    /**
     * call native Drupal AjaxCommands.
     */
    callAjaxCommands: function (model, response, options) {
      var commands = new Drupal.AjaxCommands();

      for (var i in response) {
        if (response.hasOwnProperty(i) && response[i].command && commands[response[i].command]) {
          commands[response[i].command]({getEffect: Drupal.ajax.prototype.getEffect}, response[i]);
        }
      }
    },

    /**
     * Calculate regions and blocks dimensions.
     */
    calculateDimensions: function () {
      if (Drupal.editUiBlock.collections.blockCollection) {
        // Calculate block dimensions.
        Drupal.editUiBlock.collections.blockCollection.forEach(function (block) {
          block.trigger('calculateDimensions');
        });
      }

      if (Drupal.editUiRegion.collections.regionCollection) {
        Drupal.editUiRegion.collections.regionCollection.forEach(function (region) {
          var regionOffset;
          var regionDimensions;
          var blocks;

          // Calculate region dimensions.
          region.trigger('calculateDimensions');

          regionOffset = region.get('offset');
          regionDimensions = region.get('dimensions');
          blocks = Drupal.editUiBlock.collections.blockCollection.getRegionBlocks(region.get('region'));

          // Calculate margins.
          blocks.forEach(function (block, index) {
            var prev;
            var next;
            var blockOffset = block.get('offset');
            var blockDimensions = block.get('dimensions');

            if (index === 0) {
              prev = regionOffset.top;
            }
            else {
              prev = blocks[index - 1].get('offset').top + blocks[index - 1].get('dimensions').height;
            }

            if (index === blocks.length - 1) {
              next = regionOffset.top + regionDimensions.height;
            }
            else {
              next = blocks[index + 1].get('offset').top;
            }

            block.setMargins({
              top: blockOffset.top - prev,
              bottom: next - blockOffset.top - blockDimensions.height
            });
          });
        });
      }
    },

    /**
     * Reset default states.
     */
    reset: function () {
      // Unselect and deactivate all regions.
      if (Drupal.editUiRegion.collections.regionCollection) {
        Drupal.editUiRegion.collections.regionCollection.forEach(function (region) {
          region.unselect();
          region.deactivate();
        });
      }

      // Show all blocks.
      if (Drupal.editUiBlock.collections.blockCollection) {
        Drupal.editUiBlock.collections.blockCollection.forEach(function (block) {
          block.show();
        });
      }

      // Hide dragging block.
      if (Drupal.editUiBlock.models.draggingBlockModel) {
        Drupal.editUiBlock.models.draggingBlockModel.hide();
        Drupal.editUiBlock.models.draggingBlockModel.trigger('update');
      }
    }
  };

  /**
   * Command to add new status messages.
   */
  Drupal.AjaxCommands.prototype.editUiAddMessage = function (ajax, response, status) {
    var $content = $(response.content);
    var $messageBlockContainer = $('.js-edit-ui__system_messages_block').eq(0);

    // Hide and insert.
    $content.hide().appendTo($messageBlockContainer);

    // Show and hide old messages.
    $content.fadeIn(response.speed, function () {
      $messageBlockContainer.children().not(this).fadeOut(response.speed);
    });
  };

})(Drupal, jQuery);
