/**
 * @file
 * A Backbone view for the edit_ui region element.
 */

(function (Drupal, Backbone, $) {
  "use strict";

  /**
   * Backbone view for the edit_ui region.
   */
  Drupal.editUiRegion.RegionVisualView = Backbone.View.extend({
    /**
     * Custom data.
     */
    activeClass: 'is-edit-ui-region-active',
    selectClass: 'is-edit-ui-region-selected',

    /**
     * {@inheritdoc}
     */
    initialize: function (options) {
      // Add listeners.
      this.listenTo(this.model, 'change:isActive', this.render);
      this.listenTo(this.model, 'change:isSelected', this.render);
      this.listenTo(this.model, 'init', this.initData);
      this.listenTo(this.model, 'dragOut', this.dragOut);
      this.listenTo(this.model, 'dragIn', this.dragIn);
      this.listenTo(this.model, 'drop', this.drop);
      this.listenTo(this.model, 'addBlock', this.addBlock);
      this.listenTo(this.model, 'calculateDimensions', this.calculateDimensions);

      // Get DOM elements.
      this.$block = this.$el.children('.js-edit-ui__region-block');

      // Init model state.
      this.model.setBlock(this.$block);
    },

    /**
     * {@inheritdoc}
     */
    render: function () {
      this.$el.toggleClass(this.activeClass, this.model.get('isActive'));
      this.$el.toggleClass(this.selectClass, this.model.get('isSelected'));
    },

    /**
     * DragOut event callback.
     *
     * @param Drupal.editUiBlock.BlockModel block
     *   The block model of the dragged element.
     * @param Object args
     *   The drag position.
     */
    dragOut: function (block, args) {
      var regionOffset = this.model.get('offset');
      var regionDimensions = this.model.get('dimensions');

      if (args.x > regionOffset.left &&
          args.x < regionOffset.left + regionDimensions.width &&
          args.y > regionOffset.top &&
          args.y < regionOffset.top + regionDimensions.height) {
        // Enter a region.
        this.model.activate();
      }
    },

    /**
     * DragIn event callback.
     *
     * @param Drupal.editUiBlock.BlockModel block
     *   The block model of the dragged element.
     * @param Object args
     *   The drag position.
     */
    dragIn: function (block, args) {
      var models;
      var regionOffset = this.model.get('offset');
      var regionDimensions = this.model.get('dimensions');

      if (args.x < regionOffset.left ||
          args.x > regionOffset.left + regionDimensions.width ||
          args.y < regionOffset.top ||
          args.y > regionOffset.top + regionDimensions.height) {
        // Leave the region.
        this.beforeModel = block;
        this.model.deactivate();
        Drupal.editUiBlock.models.draggingBlockModel.trigger('update');
      }
      else {
        // Manage block order inside the region.
        models = this.getAtPositionModels(args.y, 'top');
        if (models.before !== this.beforeModel) {
          this.beforeModel = models.before;
          if (models.before !== block && models.after !== block) {
            Drupal.editUiBlock.models.draggingBlockModel.trigger('update', models.before.get('block'));
          }
          else {
            Drupal.editUiBlock.models.draggingBlockModel.trigger('update');
          }
        }
      }
    },

    /**
     * Drop event callback.
     *
     * @param Drupal.editUiBlock.BlockModel block
     *   The block model of the dropped element.
     * @param Object args
     *   The drop position.
     */
    drop: function (block, args) {
      var weight;
      var region = this.model.get('region');
      var models = this.getAtPositionModels(args.y, 'top');
      var blocks = Drupal.editUiBlock.collections.blockCollection
        .getRegionBlocks(region)
        .filter(function (model) {
          return model.cid !== block.cid;
        });

      // Skip if no changes in drop position.
      if (models.before === block || models.after === block) {
        return;
      }

      // Calculate and update weights.
      if (this.beforeModel instanceof Drupal.editUiBlock.BlockModel) {
        weight = this.beforeModel.get('weight') + 1;
        block.drop(this.model.get('region'), weight);

        // We must update all following block weight.
        blocks
          .slice(blocks.indexOf(this.beforeModel) + 1)
          .forEach(function (model) {
            weight++;
            model.setWeight(weight);
          });
      }
      else if (blocks[0]) {
        block.drop(this.model.get('region'), blocks[0].get('weight') - 1);
      }
      else {
        block.drop(this.model.get('region'), 0);
      }

      // Trigger addBlock event.
      this.model.trigger('addBlock', block, block.get('block'));
    },

    /**
     * Add block event callback.
     *
     * @param Drupal.editUiBlock.BlockModel addedBlock
     *   The model of the block.
     * @param jQuery $block
     *   The block element.
     */
    addBlock: function (addedBlock, $block) {
      if (!this.beforeModel) {
        this.beforeModel = this.getAtPositionModels(addedBlock.get('weight'), 'weight').before;
      }

      // Insert block.
      this.beforeModel.get('block').after($block);
      this.beforeModel = null;
    },

    /**
     * Calculate the region dimensions.
     */
    calculateDimensions: function () {
      var offset = this.$el.offset();
      var paddingTop = this.$el.css('paddingTop');
      var paddingLeft = this.$el.css('paddingLeft');

      // Remove px unit.
      paddingTop = Number(paddingTop.substr(0, paddingTop.indexOf('px')));
      paddingLeft = Number(paddingLeft.substr(0, paddingLeft.indexOf('px')));

      // Save dimensions.
      this.model.setOffset({
        top: offset.top + paddingTop,
        left: offset.left + paddingLeft
      });
      this.model.setDimensions({
        width: this.$el.width(),
        height: this.$el.height()
      });
    },

    /**
     * Get the models representing the elements that are just before and after
     * the dragged/dropped element.
     *
     * @param Number value
     *   The top position or block weight.
     * @return jQuery
     *   The jQuery element just before the given position.
     */
    getAtPositionModels: function (value, comparator) {
      var compareValue;
      var afterModel;
      var beforeModel;
      var blocks = Drupal.editUiBlock.collections.blockCollection.getRegionBlocks(this.model.get('region'));
      var i = blocks.length - 1;

      // Fetch before Model.
      while (blocks[i] && !beforeModel) {

        if (comparator === 'top') {
          compareValue = blocks[i].get('offset').top + blocks[i].get('dimensions').height / 2;
        }
        else {
          compareValue = blocks[i].get('weight');
        }

        if (blocks[i].get('isVisible') && value > compareValue) {
          beforeModel = blocks[i];
        }
        else {
          i--;
        }
      }

      // If not found the before model is equal to the region.
      if (!beforeModel) {
        beforeModel = this.model;
        i = 0;
      }
      else {
        i++;
      }

      // Fetch after Model.
      while (blocks[i] && !afterModel) {
        if (blocks[i].get('isVisible')) {
          afterModel = blocks[i];
        }
        else {
          i++;
        }
      }

      return {
        before: beforeModel,
        after: afterModel
      };
    }
  });

}(Drupal, Backbone, jQuery));
